"""Manager and Tornado handlers for license reporting."""

# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.

import mimetypes
import json
import re
import csv
import io
from pathlib import Path

from concurrent.futures import ThreadPoolExecutor

from tornado import web, gen

from traitlets import Unicode, List
from traitlets.config import LoggingConfigurable

from .server import APIHandler
from .config import get_federated_extensions

# this is duplicated in @juptyerlab/builder
DEFAULT_THIRD_PARTY_LICENSE_FILE = "third-party-licenses.json"
UNKNOWN_PACKAGE_NAME = "UNKNOWN"

if mimetypes.guess_extension("text/markdown") is None:  # pragma: no cover
    # for python <3.8 https://bugs.python.org/issue39324
    mimetypes.add_type("text/markdown", ".md")


class LicensesManager(LoggingConfigurable):
    """A manager for listing the licenses for all frontend end code distributed
    by an application and any federated extensions
    """

    executor = ThreadPoolExecutor(max_workers=1)

    third_party_licenses_files = List(
        Unicode(),
        default_value=[DEFAULT_THIRD_PARTY_LICENSE_FILE],
        help="the license report data in built app and federated extensions",
    )

    @property
    def federated_extensions(self):
        """Lazily load the currrently-available federated extensions.

        This is expensive, but probably the only way to be sure to get
        up-to-date license information for extensions installed interactively.
        """
        labextensions_path = sum(
            [
                self.parent.labextensions_path,
                self.parent.extra_labextensions_path,
            ],
            [],
        )
        return get_federated_extensions(labextensions_path)

    @gen.coroutine
    def report_async(
        self, report_format="markdown", bundles_pattern=".*", full_text=False
    ):
        """Asynchronous wrapper around the potentially slow job of locating
        and encoding all of the licenses
        """
        report = yield self.executor.submit(
            self.report,
            report_format=report_format,
            bundles_pattern=bundles_pattern,
            full_text=full_text,
        )
        return report

    def report(self, report_format, bundles_pattern, full_text):
        """create a human- or machine-readable report"""
        bundles = self.bundles(bundles_pattern=bundles_pattern)
        if report_format == "json":
            return self.report_json(bundles), "application/json"
        elif report_format == "csv":
            return self.report_csv(bundles), "text/csv"
        elif report_format == "markdown":
            return (
                self.report_markdown(bundles, full_text=full_text),
                "text/markdown",
            )

    def report_json(self, bundles):
        """create a JSON report
        TODO: SPDX
        """
        return json.dumps({"bundles": bundles}, indent=2, sort_keys=True)

    def report_csv(self, bundles):
        """create a CSV report"""
        outfile = io.StringIO()
        fieldnames = ["name", "versionInfo", "licenseId", "extractedText"]
        writer = csv.DictWriter(outfile, fieldnames=["bundle"] + fieldnames)
        writer.writeheader()
        for bundle_name, bundle in bundles.items():
            for package in bundle["packages"]:
                writer.writerow(
                    {
                        "bundle": bundle_name,
                        **{field: package.get(field, "") for field in fieldnames},
                    }
                )
        return outfile.getvalue()

    def report_markdown(self, bundles, full_text=True):
        """create a markdown report"""
        lines = []
        library_names = [
            len(package.get("name", UNKNOWN_PACKAGE_NAME))
            for bundle_name, bundle in bundles.items()
            for package in bundle.get("packages", [])
        ]
        longest_name = max(library_names) if library_names else 1

        for bundle_name, bundle in bundles.items():
            # TODO: parametrize template
            lines += [f"# {bundle_name}", ""]

            packages = bundle.get("packages", [])
            if not packages:
                lines += ["> No licenses found", ""]
                continue

            for package in packages:
                name = package.get("name", UNKNOWN_PACKAGE_NAME).strip()
                version_info = package.get("versionInfo", UNKNOWN_PACKAGE_NAME).strip()
                license_id = package.get("licenseId", UNKNOWN_PACKAGE_NAME).strip()
                extracted_text = package.get("extractedText", "")

                lines += [
                    "## "
                    + (
                        "\t".join(
                            [
                                f"""**{name}**""".ljust(longest_name),
                                f"""`{version_info}`""".ljust(20),
                                license_id,
                            ]
                        )
                    )
                ]

                if full_text:
                    if not extracted_text:
                        lines += ["", "> No license text available", ""]
                    else:
                        lines += ["", "", "<pre/>", extracted_text, "</pre>", ""]
        return "\n".join(lines)

    def license_bundle(self, path, bundle):
        """Return the content of a packages's license bundles"""
        bundle_json = {"packages": []}

        for license_file in self.third_party_licenses_files:
            licenses_path = path / license_file
            self.log.debug("Loading licenses from %s", licenses_path)
            if not licenses_path.exists():
                self.log.warning(
                    "Third-party licenses not found for %s: %s", bundle, licenses_path
                )
                continue

            try:
                file_json = json.loads(licenses_path.read_text(encoding="utf-8"))
            except Exception as err:
                self.log.warning(
                    "Failed to open third-party licenses for %s: %s\n%s",
                    bundle,
                    licenses_path,
                    err,
                )
                continue

            try:
                bundle_json["packages"].extend(file_json["packages"])
            except Exception as err:
                self.log.warning(
                    "Failed to find packages for %s: %s\n%s",
                    bundle,
                    licenses_path,
                    err,
                )
                continue

        return bundle_json

    def app_static_info(self):
        """get the static directory for this app

        This will usually be in `static_dir`, but may also appear in the
        parent of `static_dir`.
        """
        path = Path(self.parent.static_dir)
        package_json = path / "package.json"
        if not package_json.exists():
            parent_package_json = path.parent / "package.json"
            if parent_package_json.exists():
                package_json = parent_package_json
            else:
                return None, None
        name = json.loads(package_json.read_text(encoding="utf-8"))["name"]
        return path, name

    def bundles(self, bundles_pattern=".*"):
        """Read all of the licenses
        TODO: schema
        """
        bundles = {
            name: self.license_bundle(Path(ext["ext_path"]), name)
            for name, ext in self.federated_extensions.items()
            if re.match(bundles_pattern, name)
        }

        app_path, app_name = self.app_static_info()
        if app_path is not None and re.match(bundles_pattern, app_name):
            bundles[app_name] = self.license_bundle(app_path, app_name)

        if not bundles:
            self.log.warn("No license bundles found at all")

        return bundles


class LicensesHandler(APIHandler):
    """A handler for serving licenses used by the application"""

    def initialize(self, manager: LicensesManager):
        super(LicensesHandler, self).initialize()
        self.manager = manager

    @web.authenticated
    async def get(self, _args):
        """Return all the frontend licenses"""
        full_text = bool(json.loads(self.get_argument("full_text", "true")))
        report_format = self.get_argument("format", "json")
        bundles_pattern = self.get_argument("bundles", ".*")
        download = bool(json.loads(self.get_argument("download", "0")))

        report, mime = await self.manager.report_async(
            report_format=report_format,
            bundles_pattern=bundles_pattern,
            full_text=full_text,
        )

        if download:
            filename = "{}-licenses{}".format(
                self.manager.parent.app_name.lower(), mimetypes.guess_extension(mime)
            )
            self.set_attachment_header(filename)
        self.write(report)
        await self.finish(_mime_type=mime)

    def finish(self, _mime_type, *args, **kwargs):
        """Overload the regular finish, which (sensibly) always sets JSON"""
        self.update_api_activity()
        self.set_header("Content-Type", _mime_type)
        return super(APIHandler, self).finish(*args, **kwargs)



#!/usr/bin/env python

import os
from jupyterlab_server import LabServerApp

header = """\
.. _api-full-config:


Config file and command line options
====================================

The JupyterLab Server can be run with a variety of command line arguments.
A list of available options can be found below in the :ref:`options section
<options>`.

Defaults for these options can also be set by creating a file named
``jupyter_jupyterlab_server_config.py`` in your Jupyter folder. The Jupyter
folder is in your home directory, ``~/.jupyter``.

To create a ``jupyter_jupyterlab_server_config.py`` file, with all the defaults
commented out, you can use the following command line::

  $ python -m jupyterlab_server --generate-config


.. _options:

Options
-------

This list of options can be generated by running the following and hitting
enter::

  $ python -m jupyterlab_server --help-all

"""
# Handle local and RTD locations
cwd = os.getcwd()
if os.path.basename(cwd) == 'source':
    destination = os.path.join(cwd, 'api/app-config.rst')
else:
    destination = os.path.join(cwd, 'source/api/app-config.rst')

with open(destination, 'w') as f:
    f.write(header)
    f.write(LabServerApp().document_config_options())



import json
import os
import os.path as osp
import shutil
from os.path import join as pjoin

import pytest

from jupyterlab_server import LabServerApp
from jupyterlab_server.app import LabServerApp

pytest_plugins = [
    "jupyter_server.pytest_plugin"
]

def mkdir(tmp_path, *parts):
    path = tmp_path.joinpath(*parts)
    if not path.exists():
        path.mkdir(parents=True)
    return path

app_settings_dir = pytest.fixture(lambda tmp_path: mkdir(tmp_path, 'app_settings'))
user_settings_dir = pytest.fixture(lambda tmp_path: mkdir(tmp_path, 'user_settings'))
schemas_dir = pytest.fixture(lambda tmp_path: mkdir(tmp_path, 'schemas'))
workspaces_dir = pytest.fixture(lambda tmp_path: mkdir(tmp_path, 'workspaces'))
labextensions_dir = pytest.fixture(lambda tmp_path: mkdir(tmp_path, 'labextensions_dir'))

@pytest.fixture
def make_labserver_extension_app(
    jp_root_dir,
    jp_template_dir,
    app_settings_dir,
    user_settings_dir,
    schemas_dir,
    workspaces_dir,
    labextensions_dir
    ):

    def _make_labserver_extension_app(**kwargs):

        return LabServerApp(
            static_dir = str(jp_root_dir),
            templates_dir = str(jp_template_dir),
            app_url = '/lab',
            app_settings_dir = str(app_settings_dir),
            user_settings_dir = str(user_settings_dir),
            schemas_dir = str(schemas_dir),
            workspaces_dir = str(workspaces_dir),
            extra_labextensions_path=[str(labextensions_dir)]
        )

    # Create the index files.
    index = jp_template_dir.joinpath('index.html')
    index.write_text("""
<!DOCTYPE html>
<html>
<head>
  <title>{{page_config['appName'] | e}}</title>
</head>
<body>
    {# Copy so we do not modify the page_config with updates. #}
    {% set page_config_full = page_config.copy() %}

    {# Set a dummy variable - we just want the side effect of the update. #}
    {% set _ = page_config_full.update(baseUrl=base_url, wsUrl=ws_url) %}

      <script id="jupyter-config-data" type="application/json">
        {{ page_config_full | tojson }}
      </script>
  <script src="{{page_config['fullStaticUrl'] | e}}/bundle.js" main="index"></script>

  <script type="text/javascript">
    /* Remove token from URL. */
    (function () {
      var parsedUrl = new URL(window.location.href);
      if (parsedUrl.searchParams.get('token')) {
        parsedUrl.searchParams.delete('token');
        window.history.replaceState({ }, '', parsedUrl.href);
      }
    })();
  </script>
</body>
</html>
""")

    # Copy the schema files.
    src = pjoin(
        os.path.abspath(os.path.dirname(__file__)),
        'tests',
        'schemas',
        '@jupyterlab')
    dst = pjoin(str(schemas_dir), '@jupyterlab')
    if os.path.exists(dst):
        shutil.rmtree(dst)
    shutil.copytree(src, dst)

    # Create the federated extensions
    for name in ['apputils-extension', 'codemirror-extension']:
        target_name = name + '-federated'
        target = pjoin(str(labextensions_dir), '@jupyterlab', target_name)
        src = pjoin(
            os.path.abspath(os.path.dirname(__file__)),
            'tests',
            'schemas',
            '@jupyterlab',
            name)
        dst = pjoin(target, 'schemas', '@jupyterlab', target_name)
        if osp.exists(dst):
            shutil.rmtree(dst)
        shutil.copytree(src, dst)
        with open(pjoin(target, 'package.orig.json'), 'w') as fid:
            data = dict(name=target_name, jupyterlab=dict(extension=True))
            json.dump(data, fid)

    # Copy the overrides file.
    src = pjoin(
        os.path.abspath(os.path.dirname(__file__)),
        'tests',
        'app-settings',
        'overrides.json')
    dst = pjoin(str(app_settings_dir), 'overrides.json')
    if os.path.exists(dst):
        os.remove(dst)
    shutil.copyfile(src, dst)

    # Copy workspaces.
    data = pjoin(
        os.path.abspath(os.path.dirname(__file__)),
        'tests',
        'workspaces')
    for item in os.listdir(data):
        src = pjoin(data, item)
        dst = pjoin(str(workspaces_dir), item)
        if os.path.exists(dst):
            os.remove(dst)
        shutil.copy(src, str(workspaces_dir))

    return _make_labserver_extension_app


@pytest.fixture
def labserverapp(jp_serverapp, make_labserver_extension_app):
    app = make_labserver_extension_app()
    app._link_jupyter_server_extension(jp_serverapp)
    app.initialize()
    return app



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
# import os
# import sys
# sys.path.insert(0, os.path.abspath('.'))
import os.path as osp
import shutil

HERE = osp.abspath(osp.dirname(__file__))

# -- Project information -----------------------------------------------------

project = 'JupyterLab Server'
copyright = '2021, Project Jupyter'
author = 'Project Jupyter'

# The full version, including alpha/beta/rc tags
_version_py = osp.join(HERE, '..', '..', 'jupyterlab_server', '_version.py')
version_ns = {}

with open(_version_py, mode='r') as version_file:
    exec(version_file.read(), version_ns)

# The short X.Y version.
version = '%i.%i' % version_ns['version_info'][:2]
# The full version, including alpha/beta/rc tags.
release = version_ns['__version__']

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'myst_parser',
    'numpydoc',
    'autodoc_traits',
    'sphinx.ext.autodoc',
    'sphinx.ext.intersphinx',
    'sphinxcontrib.openapi',
    'sphinx.ext.napoleon',
    'sphinx.ext.mathjax',
    'sphinx_copybutton'
]

myst_enable_extensions = ["html_image"]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
#
html_theme = "pydata_sphinx_theme"

# Add an Edit this Page button
html_theme_options = {
    "use_edit_page_button": True,
}

# Output for github to be used in links
html_context = {
    "github_user": "jupyterlab",  # Username
    "github_repo": "jupyterlab_server",  # Repo name
    "github_version": "master",  # Version
    "doc_path": "/docs/source/",  # Path in the checkout to the docs root
}

# This option generates errors when methods do not have docstrings,
# so disable
numpydoc_show_class_members = False

def setup(app):
    dest = osp.join(HERE, 'changelog.md')
    shutil.copy(osp.join(HERE, '..', '..', 'CHANGELOG.md'), dest)
    with open(osp.join(HERE, '../autogen_config.py')) as f:
        exec(compile(f.read(), '../autogen_config.py', 'exec'), {})




from .utils import validate_request


async def test_get_listing(jp_fetch, labserverapp):
    url = r"lab/api/listings/@jupyterlab/extensionmanager-extension/listings.json"
    r = await jp_fetch(*url.split('/'))
    validate_request(r)



"""Test the kernels service API."""
import json
import os
import pytest
import shutil
import tornado

from strict_rfc3339 import rfc3339_to_timestamp

from .utils import expected_http_error, maybe_patch_ioloop, big_unicode_string
from .utils import validate_request

maybe_patch_ioloop()


async def test_delete(jp_fetch, labserverapp):
    orig = 'f/o/o'
    copy = 'baz'
    r = await jp_fetch('lab', 'api', 'workspaces', orig)
    validate_request(r)
    res = r.body.decode()
    data = json.loads(res)
    data['metadata']['id'] = copy
    r2 = await jp_fetch('lab', 'api', 'workspaces', copy,
        method='PUT',
        body=json.dumps(data))
    assert r2.code == 204
    r3 = await jp_fetch('lab', 'api', 'workspaces', copy,
        method='DELETE',
        )
    assert r3.code == 204


async def test_get_non_existant(jp_fetch, labserverapp):
    id = 'foo'

    r = await jp_fetch('lab', 'api', 'workspaces', id)
    validate_request(r)
    data = json.loads(r.body.decode())

    r2 = await jp_fetch('lab', 'api', 'workspaces', id,
        method='PUT',
        body=json.dumps(data))
    validate_request(r2)

    r3 = await jp_fetch('lab', 'api', 'workspaces', id)
    validate_request(r3)
    data = json.loads(r3.body.decode())
    first_metadata = data["metadata"]
    first_created = rfc3339_to_timestamp(first_metadata['created'])
    first_modified = rfc3339_to_timestamp(first_metadata['last_modified'])

    r4 = await jp_fetch('lab', 'api', 'workspaces', id,
        method='PUT',
        body=json.dumps(data))
    validate_request(r4)

    r5 = await jp_fetch('lab', 'api', 'workspaces', id)
    validate_request(r5)
    data = json.loads(r5.body.decode())
    second_metadata = data["metadata"]
    second_created = rfc3339_to_timestamp(second_metadata['created'])
    second_modified = rfc3339_to_timestamp(second_metadata['last_modified'])

    assert first_created <= second_created
    assert first_modified < second_modified


@pytest.mark.skipif(os.name == "nt", reason="Temporal failure on windows")
async def test_get(jp_fetch, labserverapp):
    id = 'foo'
    r = await jp_fetch('lab', 'api', 'workspaces', id)
    validate_request(r)
    data = json.loads(r.body.decode())
    metadata = data['metadata']
    assert metadata['id'] == id
    assert rfc3339_to_timestamp(metadata['created'])
    assert rfc3339_to_timestamp(metadata['last_modified'])

    r2 = await jp_fetch('lab', 'api', 'workspaces', id)
    validate_request(r2)
    data = json.loads(r.body.decode())
    assert data['metadata']['id'] == id

async def test_listing(jp_fetch, labserverapp):
    # ID fields are from workspaces/*.jupyterlab-workspace
    listing = set(['foo', 'f/o/o/'])
    r = await jp_fetch('lab', 'api', 'workspaces/')
    validate_request(r)
    res = r.body.decode()
    data = json.loads(res)
    output = set(data['workspaces']['ids'])
    assert output == listing


async def test_listing_dates(jp_fetch, labserverapp):
    r = await jp_fetch('lab', 'api', 'workspaces')
    data = json.loads(r.body.decode())
    values = data['workspaces']['values']
    times = sum(
        [
            [ws['metadata'].get('last_modified'), ws['metadata'].get('created')]
            for ws in values
        ],
        []
    )
    assert None not in times
    [rfc3339_to_timestamp(t) for t in times]


async def test_put(jp_fetch, labserverapp):
    id = 'foo'
    r = await jp_fetch('lab', 'api', 'workspaces', id)
    assert r.code == 200
    res = r.body.decode()
    data = json.loads(res)
    data["metadata"]["big-unicode-string"] = big_unicode_string[::-1]
    r2 = await jp_fetch('lab', 'api', 'workspaces', id,
        method='PUT',
        body=json.dumps(data)
        )
    assert r2.code == 204


async def test_bad_put(jp_fetch, labserverapp):
    orig = 'foo'
    copy = 'bar'
    r = await jp_fetch('lab', 'api', 'workspaces', orig)
    assert r.code == 200
    res = r.body.decode()
    data = json.loads(res)
    with pytest.raises(tornado.httpclient.HTTPClientError) as e:
        await jp_fetch('lab', 'api', 'workspaces', copy,
            method='PUT',
            body=json.dumps(data)
        )
    assert expected_http_error(e, 400)


async def test_blank_put(jp_fetch, labserverapp):
    orig = 'foo'
    r = await jp_fetch('lab', 'api', 'workspaces', orig)
    assert r.code == 200
    res = r.body.decode()
    data = json.loads(res)
    with pytest.raises(tornado.httpclient.HTTPClientError) as e:
        await jp_fetch('lab', 'api', 'workspaces',
            method='PUT',
            body=json.dumps(data)
        )
    assert expected_http_error(e, 400)



"""Test the Settings service API.
"""
import io
import csv
import json
import mimetypes

import pytest
import mistune

from .. import LicensesApp
from ..licenses_handler import DEFAULT_THIRD_PARTY_LICENSE_FILE

# utilities

FULL_ENTRY = (
    ("name", "@jupyterlab/foo"),
    ("versionInfo", "0.0.1"),
    ("licenseId", "BSD-3-Clause"),
    ("extractedText", "> license text goes here"),
)


def _read_csv(csv_text):
    with io.StringIO() as csvfile:
        csvfile.write(csv_text)
        csvfile.seek(0)
        return [*csv.DictReader(csvfile)]


def _make_static_dir(
    app, tmp_path, has_licenses=True, license_json=None, package_in_app=False
):
    app_dir = tmp_path / "app"
    static_dir = app_dir / "static"
    static_dir.mkdir(parents=True)

    package_text = json.dumps({"name": "@jupyterlab/top", "version": "0.0.1"})
    package_json = (app_dir if package_in_app else static_dir) / "package.json"
    package_json.write_text(package_text, encoding="utf-8")

    if has_licenses:
        (static_dir / DEFAULT_THIRD_PARTY_LICENSE_FILE).write_text(
            license_json or _good_license_json(),
            encoding="utf-8",
        )

    setattr(app, "static_dir", str(static_dir))


def _good_license_json():
    return json.dumps(
        {"packages": [dict(FULL_ENTRY[:i]) for i in range(1 + len(FULL_ENTRY))]}
    )


@pytest.fixture(
    params=[
        ["application/json", "json", json.loads],
        ["text/csv", "csv", _read_csv],
        ["text/markdown", "markdown", mistune.markdown],
    ]
)
def mime_format_parser(request):
    return request.param


@pytest.fixture(params=[True, False])
def has_licenses(request):
    return request.param


@pytest.fixture
def licenses_app(tmp_path, has_licenses):
    app = LicensesApp()
    _make_static_dir(app, tmp_path, has_licenses)
    return app


# the actual tests


@pytest.mark.parametrize("has_static_dir", [True, False])
@pytest.mark.parametrize("full_text", ["true", "false"])
@pytest.mark.parametrize("bundles_pattern", ["", "@jupyterlab/.*", "nothing"])
async def test_get_license_report(
    mime_format_parser,
    has_static_dir,
    has_licenses,
    full_text,
    bundles_pattern,
    jp_fetch,
    labserverapp,
    tmp_path,
):
    if has_static_dir:
        _make_static_dir(labserverapp, tmp_path, has_licenses)
    mime, fmt, parse = mime_format_parser
    params = {"format": fmt, "full_text": full_text}
    if bundles_pattern:
        params["bundles"] = bundles_pattern
    r = await jp_fetch("lab", "api", "licenses/", params=params)
    assert r.code == 200
    assert r.headers["Content-type"] == mime
    res = r.body.decode()
    assert parse(res) is not None


async def test_download_license_report(
    jp_fetch,
    labserverapp,
    mime_format_parser,
):
    mime, fmt, parse = mime_format_parser
    params = {"format": fmt, "download": "1"}
    r = await jp_fetch("lab", "api", "licenses/", params=params)
    assert r.code == 200
    assert r.headers["Content-type"] == mime
    extension = mimetypes.guess_extension(mime)
    assert extension, f"no extension guessed for {mime}"
    assert extension in r.headers["Content-Disposition"], f"{r.headers}"


async def test_dev_mode_license_report(
    jp_fetch,
    labserverapp,
    tmp_path,
):
    _make_static_dir(labserverapp, tmp_path, package_in_app=True)
    r = await jp_fetch("lab", "api", "licenses/")
    assert r.code == 200


@pytest.mark.parametrize(
    "license_json",
    [
        "// leading comment\n" + _good_license_json(),
        _good_license_json().replace("packages", "whatever"),
    ],
)
async def test_malformed_license_report(
    license_json,
    mime_format_parser,
    jp_fetch,
    labserverapp,
    tmp_path,
):
    _make_static_dir(labserverapp, tmp_path, license_json=license_json)
    mime, fmt, parse = mime_format_parser
    r = await jp_fetch("lab", "api", "licenses/")
    assert r.code == 200


async def test_licenses_cli(licenses_app, capsys, mime_format_parser):
    mime, fmt, parse = mime_format_parser
    args = []
    if fmt != "markdown":
        args += [f"--{fmt}"]
    licenses_app.initialize(args)

    with pytest.raises(SystemExit) as exited:
        licenses_app.start()

    assert exited.type == SystemExit
    assert exited.value.code == 0

    captured = capsys.readouterr()
    assert parse(captured.out) is not None
